local xw = require 'extraway'
xw.addPackage(".storageUtils")

local setAttachmentInfo = function(record, setSize, fpMode)
   xw.log.msg("Entering attachUtils.setAttachmentInfo()")
   local attachments = record:selectNodes("//*[name()='xw:file']")
   local done = false
   if ( attachments:isValid() and 0 < #attachments) then
      local singleFile = attachments:getFirst()
      while ( singleFile:isValid() ) do
         local name = nil -- In order to load attachment id and convert it into attachment name only if needed
         local doSize = false
         local doHash = false
         local size = ""
         local impronta = ""
         local tipoImpronta = ""
      
         if ( setSize ) then -- Need to set the attachment size
           size = singleFile:getAttribute("size")
           if ( "" == size ) then
              doSize = true
           end
         end

         if ( "" ~= fpMode ) then -- Need to set the attachment finger print 
           local der_from = singleFile:getAttribute("der_from")
           if ( "" == der_from ) then
              -- Primary attachment, not a derived one.
              impronta = singleFile:getAttribute("impronta")
              tipoImpronta = singleFile:getAttribute("tipoImpronta")
              if ( "" == impronta or fpMode ~= tipoImpronta ) then
                 doHash =true
              end
           end

           -- Remove old finterprint only if the new one is required
           local oldFingerPrint = record:selectFirstNode("/doc/impronta")
           if ( oldFingerPrint:isValid() ) then
              oldFingerPrint:erase()
              done = true
           end
         end

        if ( doSize or doHash ) then
           name = singleFile:getAttribute("name")
           name = xw.getAttachmentFileName(name)
           -- Two ways, local file and remote repository
           if ( xw.utils.fileExist(name) ) then
               -- Local
               if ( doSize ) then
                  singleFile:setAttribute("size", tostring(xw.utils.getFileSize(name)))
                  done = true
               end
               if ( doHash ) then
                  local fingerPrint = xw.calcDigest(fpMode, name)
                  singleFile:setAttribute("impronta", fingerPrint)
                  singleFile:setAttribute("tipoImpronta", fpMode)
                  done = true
               end
           else
             if ( storageUtils.isOn() ) then
               -- Remote
               local remoteName = singleFile:getAttribute("name")
               while ( "0" == remoteName:sub(1,1) ) do
                  remoteName = remoteName:sub(2)
               end
               remoteName = remoteName:lower()
               impronta, size = storageUtils.getRemoteFileHash(remoteName, fpMode)
               if ( doSize and "" ~= size and "0" ~= size ) then
                  singleFile:setAttribute("size", size)
                  done = true
               end
               if ( doHash and "" ~= impronta ) then
                  singleFile:setAttribute("impronta", impronta)
                  singleFile:setAttribute("tipoImpronta", fpMode)
                  done = true
               end
             else
              xw.log.warning("Attachment "..name.." not available locally nor remotely")
             end
           end
        end
         
        singleFile = attachments:getNext()
      end
   end
   -- xw.log.msg("Record changed in: "..record:toString())
   xw.log.msg("Exiting attachUtils.setAttachmentInfo("..tostring(done)..")")
   return record, done
end


attachUtils = {
   setAttachmentInfo = setAttachmentInfo
}