local xw = require 'extraway'
local http   = require("socket.http")
local ltn12   = require("ltn12")
xw.addPackage(".urlEncDec")
xw.addPackage(".storageUtilsCfg")

local isOn = function()
  if ( nil ~= storageUtilsCfg.host_and_port and "" ~= storageUtilsCfg.host_and_port ) then return true end
  return false
end

local isHttpValidResponse = function(code)
  if ( nil ~= code and  "number" == type(code) ) then
    if ( code >= 200 and code < 300 ) then return true end
  end
  return false
end

local normalizeFileId = function(name)
  while ( '0' == name:sub(1,1) ) do
    name = name:sub(2)
  end
  return name
end

local getRemoteFileMetadata = function(fileId)
  -- 1: convert file name into Media id
  local hdr = {}
  hdr["Accept"] = "application/json"
  local httpResult ={}
  local url = "http://"..storageUtilsCfg.host_and_port.."/media/search/findByName?name="..urlEncDec.urlencode(normalizeFileId(fileId))
  -- xw.log.msg("Url: "..url)
  local r, c, h = http.request{url=url, headers = hdr, sink=ltn12.sink.table(httpResult)}
  if ( isHttpValidResponse(c) ) then
    return httpResult[1]
  end
  return nil
end

local getRemoteFileHash = function(fileId, hashmode)
  local size = 0
  local hash = ""
  hashmode = hashmode:lower()
  local httpResult1 = getRemoteFileMetadata(fileId)
  if ( nil ~= httpResult1 ) then
    -- xw.log.msg("httpResult: "..httpResult1[1])
     
    -- 2: get the id, hash and size out of the box
    local decoded = xw.json.decode(httpResult1)
    -- for k,v in pairs(decoded._embedded.media[1]) do
    --    xw.log.msg("K: "..k.." V: "..type(v))
    -- end
    size = decoded._embedded.media[1].contentLength -- Test if nil!
    local hashType = decoded._embedded.media[1].hashType -- Test if nil!
    hash = decoded._embedded.media[1].hash -- Test if nil!
    if ( "userdata" == type(hash) or "userdata" == type(hashType) ) then
      hash = ""
    else
      if ( nil == hash ) then hash = "" end
    end
    xw.log.msg("Hash: "..hash.."; Size: "..size)
     
    if ( "" ~= hash ) then
      -- 3 Get the hash value out of the box
      if ( hashmode ~= hashType ) then
         hash = "" -- No valid value
      end
    end
        
    if ( "" == hash ) then
      -- 4 Ask for the hash value
      local id = decoded._embedded.media[1].id -- test if nil!
      xw.log.msg("Id: "..id)
  
      local hdr2 = {}
      hdr2["Accept"] = "application/it.zeroics.hash;hash-kind="..hashmode..";out-kind=base64"
  
      local httpResult2 ={}
      local r, c, h = http.request{url="http://"..storageUtilsCfg.host_and_port.."/media/"..id, headers = hdr2, sink=ltn12.sink.table(httpResult2)}
      if ( isHttpValidResponse(c) ) then
        hash = table.concat(httpResult2)
        xw.log.msg("Hash(2): "..hash)
      end
    end
  end
  return hash, tostring(size)
end

local getRemoteFile = function(fileId, fileName)
  local httpResult1 = getRemoteFileMetadata(fileId)
  if ( nil ~= httpResult1 ) then
    -- xw.log.msg("httpResult: "..httpResult1)
     
    -- 2: get the id, hash and size out of the box
    local decoded = xw.json.decode(httpResult1)
    -- for k,v in pairs(decoded._embedded.media[1]) do
    --    xw.log.msg("K: "..k.." V: "..type(v))
    -- end
    
    if (  nil == decoded._embedded.media or nil == decoded._embedded.media[1] ) then
      return false
    end
    
    local id = decoded._embedded.media[1].id -- Test if nil!
    if ( "userdata" == type(id)  ) then
      id = ""
    else
      if ( nil == id ) then id = "" end
    end
    -- xw.log.msg("Id: "..id)

    if ( "" ~= id ) then
      local httpResult2 ={}
      local url1 = "http://"..storageUtilsCfg.host_and_port.."/media/"..id
      local handle = io.open(fileName, "w+b")
      local r, c, h = http.request{url=url1, headers = {}, sink=ltn12.sink.file(handle)}
      -- io.close(handle) Already closed by http.request
      if ( isHttpValidResponse(c) ) then
        return true
      end
    end
  end
  return false
end

storageUtils = {
   isOn = isOn,
   getRemoteFileHash = getRemoteFileHash,
   getRemoteFile = getRemoteFile
}